@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2018 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -30,22 +30,20 @@ import os
 import warnings
 
 import sqlalchemy as sa
+from sqlalchemy.orm import sessionmaker, scoped_session
 
-import rattail.db
 from rattail.config import make_config
 from rattail.exceptions import ConfigurationError
-from rattail.db.util import get_engines
-from rattail.db.continuum import configure_versioning
 from rattail.db.types import GPCType
 
-import formalchemy as fa
 from pyramid.config import Configurator
 from pyramid.authentication import SessionAuthenticationPolicy
+from zope.sqlalchemy import register
 
 import tailbone.db
 from tailbone.auth import TailboneAuthorizationPolicy
-from tailbone.forms import renderers
-from tailbone.forms.alchemy import TemplateEngine
+
+from tailbone.util import get_effective_theme, get_theme_template_path
 
 
 def make_rattail_config(settings):
@@ -59,41 +57,31 @@ def make_rattail_config(settings):
         # available for web requests later
         path = settings.get('rattail.config')
         if not path or not os.path.exists(path):
-            path = settings.get('edbob.config')
-            if not path or not os.path.exists(path):
-                raise ConfigurationError("Please set 'rattail.config' in [app:main] section of config "
-                                         "to the path of your config file.  Lame, but necessary.")
-            warnings.warn("[app:main] setting 'edbob.config' is deprecated; "
-                          "please use 'rattail.config' setting instead",
-                          DeprecationWarning)
+            raise ConfigurationError("Please set 'rattail.config' in [app:main] section of config "
+                                     "to the path of your config file.  Lame, but necessary.")
         rattail_config = make_config(path)
         settings['rattail_config'] = rattail_config
     rattail_config.configure_logging()
 
-    rattail_engines = settings.get('rattail_engines')
-    if not rattail_engines:
+    # configure database sessions
+    if hasattr(rattail_config, 'rattail_engine'):
+        tailbone.db.Session.configure(bind=rattail_config.rattail_engine)
+    if hasattr(rattail_config, 'trainwreck_engine'):
+        tailbone.db.TrainwreckSession.configure(bind=rattail_config.trainwreck_engine)
+    if hasattr(rattail_config, 'tempmon_engine'):
+        tailbone.db.TempmonSession.configure(bind=rattail_config.tempmon_engine)
 
-        # Load all Rattail database engines from config, and store in settings
-        # dict.  This is necessary e.g. in the case of a host server, to have
-        # access to its subordinate store servers.
-        rattail_engines = get_engines(rattail_config)
-        settings['rattail_engines'] = rattail_engines
-
-    # Configure the database session classes.  Note that most of the time we'll
-    # be using the Tailbone Session, but occasionally (e.g. within batch
-    # processing threads) we want the Rattail Session.  The reason is that
-    # during normal request processing, the Tailbone Session is preferable as
-    # it includes Zope Transaction magic.  Within an explicitly-spawned thread
-    # however, this is *not* desirable.
-    rattail.db.Session.configure(bind=rattail_engines['default'])
-    tailbone.db.Session.configure(bind=rattail_engines['default'])
+    # create session wrappers for each "extra" Trainwreck engine
+    for key, engine in rattail_config.trainwreck_engines.items():
+        if key != 'default':
+            Session = scoped_session(sessionmaker(bind=engine))
+            register(Session)
+            tailbone.db.ExtraTrainwreckSessions[key] = Session
 
     # Make sure rattail config object uses our scoped session, to avoid
     # unnecessary connections (and pooling limits).
     rattail_config._session_factory = lambda: (tailbone.db.Session(), False)
 
-    # Configure (or not) Continuum versioning.
-    configure_versioning(rattail_config)
     return rattail_config
 
 
@@ -103,7 +91,12 @@ def provide_postgresql_settings(settings):
     this enables retrying transactions a second time, in an attempt to
     gracefully handle database restarts.
     """
-    settings.setdefault('tm.attempts', 2)
+    try:
+        import pyramid_retry
+    except ImportError:
+        settings.setdefault('tm.attempts', 2)
+    else:
+        settings.setdefault('retry.attempts', 2)
 
 
 class Root(dict):
@@ -117,44 +110,61 @@ class Root(dict):
         self.request = request
 
 
-def make_pyramid_config(settings):
+def make_pyramid_config(settings, configure_csrf=True):
     """
     Make a Pyramid config object from the given settings.
     """
-    config = Configurator(settings=settings, root_factory=Root)
+    config = settings.pop('pyramid_config', None)
+    if config:
+        config.set_root_factory(Root)
+    else:
 
-    # Configure user authentication / authorization.
-    config.set_authentication_policy(SessionAuthenticationPolicy())
+        # we want the new themes feature!
+        establish_theme(settings)
+
+        settings.setdefault('pyramid_deform.template_search_path', 'tailbone:templates/deform')
+        config = Configurator(settings=settings, root_factory=Root)
+
+    # configure user authorization / authentication
     config.set_authorization_policy(TailboneAuthorizationPolicy())
+    config.set_authentication_policy(SessionAuthenticationPolicy())
+
+    # always require CSRF token protection
+    if configure_csrf:
+        config.set_default_csrf_options(require_csrf=True, token='_csrf')
 
     # Bring in some Pyramid goodies.
-    config.include('pyramid_beaker')
+    config.include('tailbone.beaker')
+    config.include('pyramid_deform')
     config.include('pyramid_mako')
     config.include('pyramid_tm')
 
+    # bring in the pyramid_retry logic, if available
+    # TODO: pretty soon we can require this package, hopefully..
+    try:
+        import pyramid_retry
+    except ImportError:
+        pass
+    else:
+        config.include('pyramid_retry')
+
     # Add some permissions magic.
     config.add_directive('add_tailbone_permission_group', 'tailbone.auth.add_permission_group')
     config.add_directive('add_tailbone_permission', 'tailbone.auth.add_permission')
 
-    # TODO: This can finally be removed once all CRUD/index views have been
-    # converted to use the new master view etc.
-    for label, perms in settings.get('edbob.permissions', []):
-        groupkey = label.lower().replace(' ', '_')
-        config.add_tailbone_permission_group(groupkey, label)
-        for key, label in perms:
-            config.add_tailbone_permission(groupkey, key, label)
-
-    # Configure FormAlchemy.
-    fa.config.engine = TemplateEngine()
-    fa.FieldSet.default_renderers[sa.Boolean] = renderers.YesNoFieldRenderer
-    fa.FieldSet.default_renderers[sa.Date] = renderers.DateFieldRenderer
-    fa.FieldSet.default_renderers[sa.DateTime] = renderers.DateTimeFieldRenderer
-    fa.FieldSet.default_renderers[sa.Time] = renderers.TimeFieldRenderer
-    fa.FieldSet.default_renderers[GPCType] = renderers.GPCFieldRenderer
-
     return config
 
 
+def establish_theme(settings):
+    rattail_config = settings['rattail_config']
+
+    theme = get_effective_theme(rattail_config)
+    settings['tailbone.theme'] = theme
+
+    path = get_theme_template_path(rattail_config)
+    settings['mako.directories'].insert(0, path)
+
+
 def configure_postgresql(pyramid_config):
     """
     Add some PostgreSQL-specific tweaks to the final app config.  Specifically,
